set define off
declare
  im_id         numeric(16);
  im_name       varchar(256) := 'AGC';

  fc_id         numeric(16);
  fc_name       varchar(256) := 'system';
  
  fevt_id       numeric(16);
  fevt_name     varchar(256) := 'TARGET';
  
  f_id          numeric(16);
  f_name        varchar(256) := 'PERMISSION_REMOVE';

  f_lob         clob := 'package tREMOVE_;
'||'
'||'import com.engiweb.logger.impl.Log4JImpl;
'||'import com.engiweb.pm.entity.BeanList;
'||'import com.engiweb.profilemanager.backend.dao.db.SQLH;
'||'import com.engiweb.profilemanager.common.bean.AccountBean;
'||'import com.engiweb.profilemanager.common.bean.Block;
'||'import com.engiweb.profilemanager.common.bean.OrgUnitBean;
'||'import com.engiweb.profilemanager.common.bean.UserBean;
'||'import com.engiweb.profilemanager.common.bean.entitlement.EntitlementBean;
'||'import com.engiweb.profilemanager.common.bean.event.EntStateBean;
'||'import com.engiweb.profilemanager.common.bean.event.EventTargetBean;
'||'import com.engiweb.profilemanager.common.bean.rule.SyncStateBean;
'||'import com.engiweb.profilemanager.common.ruleengine.action.EntitlementAction;
'||'import com.engiweb.profilemanager.common.ruleengine.action.OrgUnitAction;
'||'import com.engiweb.profilemanager.common.ruleengine.action.UserAction;
'||'import com.engiweb.toolkit.common.DBMSException;
'||'import com.engiweb.profilemanager.backend.business.direct.UserDirect
'||'import com.engiweb.profilemanager.common.enumerations.EntitlementType;
'||'import com.engiweb.ruleengine.common.errorhandling.VetoException;
'||'
'||'global com.engiweb.profilemanager.backend.dao.db.SQLH sql;
'||'global com.engiweb.logger.impl.Log4JImpl logger;
'||'
';
 
  fr_id         numeric(16);
  fr_name       varchar(256) := 'Remove Permission from User Account';
  fr_desc       varchar(512) := '[ V1.1 - 2019-01-11 ]'; 
  
  fr_lob        clob := 'rule "Remove Permission from User Account"
'||'when
'||'	event : EventTargetBean(  )
'||'	userBean : UserBean(  )
'||'	syncBean : SyncStateBean(  )
'||'	accountBean : AccountBean(  )
'||'then
'||'
'||'  // [ V1.1 - 2019-01-11 ]
'||'
'||'	if (event.getLevel() == EventTargetBean.USER_PROFILE_FOUND) {
'||'		UserDirect userDirect = new UserDirect();
'||'
'||'		EntitlementBean entBean = syncBean.getProfile();
'||'
'||'		// Entitlements obtained by this find have the field employmentId populated
'||'		BeanList<EntitlementBean> blFound = userDirect.findEntitlement(entBean, null, accountBean, null, userBean, null, "1", sql);
'||'
'||'		userDirect.removeEntitlement(blFound, null, userBean, sql);
'||'
'||'		boolean vetoexception = false;
'||'
'||'		if (syncBean.getSyncEntitlements() != null && syncBean.getSyncEntitlements().size() > 0) {
'||'			if (syncBean.getSyncEntitlements().size() > 1 || (syncBean.getSyncEntitlements().size() == 1 && syncBean.getSyncEntitlements().get(0).getId()!=null && !syncBean.getSyncEntitlements().get(0).getId().equals(entBean.getId()))) {
'||'				vetoexception = true;
'||'			}
'||'		}
'||'
'||'		if (!vetoexception && syncBean.getSyncDelegateEntitlements() != null && syncBean.getSyncDelegateEntitlements().size() > 0) {
'||'			for (EntitlementBean e : syncBean.getSyncDelegateEntitlements()) {
'||'				if (!EntitlementType.PERMISSION.getCode().equals(e.getType())) {
'||'					vetoexception = true;
'||'				}
'||'			}
'||'		}
'||'
'||'		if (!vetoexception && ((syncBean.getCompatibleEntitlements() != null && syncBean.getCompatibleEntitlements().size() > 0) || (syncBean.getCompatileDelegateEntitlements() != null && syncBean.getCompatileDelegateEntitlements().size() > 0))) {
'||'			vetoexception = true;
'||'		}
'||'
'||'		if (vetoexception) {
'||'			event.setState(2L);
'||'			event.setTrace("The user has the permission through other roles");
'||'			throw new VetoException("The user has the permission through other roles");
'||'		}
'||'
'||'		logger.debug("Permission " + entBean.getName() + " removed from User " + userBean.getCode());
'||'	}  
'||'
'||'end
'||'
';

  counter       numeric(16);
begin
  select count(1) into counter from ideas_module t where lower ( t.code ) = lower ( im_name );
  if ( counter = 1 ) then
    select t.id into im_id from ideas_module t where lower ( t.code ) = lower ( im_name );
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO IDEAS MODULE FOUND' );
  end if;

  select count(1) into counter from flow_context t where lower ( t.name ) = lower ( fc_name );
  if ( counter = 1 ) then
    select t.id into fc_id from flow_context t where lower ( t.name ) = lower ( fc_name );
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO FLOW CONTEXT FOUND' );
  end if;

  select count(1) into counter from flow_event_type t
  where t.flow_context = fc_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fevt_name );
  if ( counter = 1 ) then
    select t.id into fevt_id from flow_event_type t
    where t.flow_context = fc_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fevt_name );
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO FLOW EVENT TYPE FOUND' );
  end if;
  
  select count(1) into counter from flow t
  where t.ideas_module = im_id and t.flow_event_type = fevt_id and lower ( t.name ) = lower ( f_name ); 
  if ( counter = 1 ) then
    select t.id into f_id from flow t
    where t.ideas_module = im_id and t.flow_event_type = fevt_id and lower ( t.name ) = lower ( f_name );
	
	update flow set flow_lob = f_lob where id = f_id;
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO FLOW FOUND' );
  end if;

  select count(1) into counter from flow_rule t where t.flow = f_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fr_name );
  if ( counter = 0 ) then
    select flow_rule_seq.nextval into fr_id from dual;
    insert into flow_rule ( id, flow, name, description, rule_lob, ideas_module )
    values ( fr_id, f_id, fr_name, fr_desc, fr_lob, im_id );
  else
    select t.id into fr_id from flow_rule t where t.flow = f_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fr_name );

    update flow_rule set description = fr_desc where id = fr_id;

    update flow_rule set rule_lob = fr_lob where id = fr_id;
  end if;

end;
/
commit;
set define on
